﻿using System;
using System.ComponentModel;
using System.IO;
using System.Net;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Windows;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using PPTShow;

namespace ShowController
{
    public partial class MainPage : PhoneApplicationPage, INotifyPropertyChanged
    {
        string _iPSelected;
        public string IPSelected
        {
            get { return this._iPSelected; }

            set
            {
                if (this._iPSelected == value)
                {
                    return;
                }

                this._iPSelected = value;
                OnPropertyChanged("IPSelected");
            }
        }

        int _port;
        public int Port
        {
            get { return this._port; }

            set
            {
                if (this._port == value)
                {
                    return;
                }

                this._port = value;
                OnPropertyChanged("Port");
            }
        }

        public MainPage()
        {
            InitializeComponent();
            this.DataContext = this;

            this.Port = 5022;
        }

        private void btnConnectClicked(object sender, RoutedEventArgs e)
        {
            string url = string.Format("http://{0}:{1}/getSnapshot", this.IPSelected, this.Port);
            Uri uri = new Uri(url);

            WebClient wc = new WebClient();

            wc.Headers[HttpRequestHeader.IfModifiedSince] = DateTime.UtcNow.ToString();
            wc.DownloadStringCompleted += new DownloadStringCompletedEventHandler(SnapshotCompleted);
            wc.DownloadStringAsync(uri);
        }

        void SnapshotCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.ToString());
            }
            else
            {
                try
                {
                    if (string.IsNullOrEmpty(e.Result) == false)
                    {
                        MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(e.Result));
                        DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(PPTDocument));
                        PPTDocument document = serializer.ReadObject(ms) as PPTDocument;

                        PhoneApplicationService.Current.State["Document"] = document;

                        Uri uri = new Uri(string.Format("/PPTController.xaml?ip={0}&port={1}",
                            this.IPSelected, this.Port), UriKind.Relative);
                        this.NavigationService.Navigate(uri);
                    }
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged == null)
            {
                return;
            }

            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}