﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using Microsoft.Phone;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using PPTShow;

namespace ShowController
{
    public partial class PPTController : PhoneApplicationPage
    {
        string _baseUrl;
        Border oldBorder = null;

        public PPTController()
        {
            InitializeComponent();
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            PPTDocument document = null;

            if (PhoneApplicationService.Current.State.ContainsKey("Document") == true)
            {
                document = PhoneApplicationService.Current.State["Document"] as PPTDocument;
            }

            if (document == null)
            {
                return;
            }

            string ipAddress;
            string port;
            NavigationContext.QueryString.TryGetValue("ip", out ipAddress);
            NavigationContext.QueryString.TryGetValue("port", out port);

            if (string.IsNullOrEmpty(ipAddress) == true ||
                string.IsNullOrEmpty(port) == true)
            {
                return;
            }

            _baseUrl = string.Format("http://{0}:{1}", ipAddress, port);

            LoadDocument(document);

            base.OnNavigatedTo(e);
        }

        private void LoadDocument(PPTDocument document)
        {
            int slideIndex = 1;
            foreach (var item in document.List)
            {
                SlideItemData tagData = new SlideItemData();
                tagData.SlideIndex = slideIndex;

                byte[] jpegContents = Convert.FromBase64String(item.ImageAsText);
                Image img = new Image();
                img.Tap += img_Tap;
                img.Tag = tagData;

                MemoryStream ms = new MemoryStream(jpegContents);
                WriteableBitmap bmp = PictureDecoder.DecodeJpeg(ms, 100, 100);
                img.Source = bmp;

                Border border = new Border();
                border.Child = img;
                border.BorderBrush = new SolidColorBrush(Colors.Yellow);

                if (slideIndex == 1)
                {
                    // 처음 슬라이드를 선택 표시
                    border.BorderThickness = new Thickness(3);
                    oldBorder = border;
                }

                imageList.Items.Add(border);

                // 파노라마 콘트롤에 추가
                PanoramaItem panItem = new PanoramaItem();
                panItem.Tag = tagData;
                Grid grid = new Grid();

                grid.RowDefinitions.Add(new RowDefinition());
                grid.RowDefinitions.Add(new RowDefinition());

                ms.Position = 0;
                bmp = PictureDecoder.DecodeJpeg(ms, document.Width, document.Height);

                img = new Image();
                img.Source = bmp;
                Grid.SetRow(img, 0);

                TextBox txt = new TextBox();
                txt.Text = item.Note;
                txt.IsReadOnly = true;
                Grid.SetRow(txt, 1);

                grid.Children.Add(img);
                grid.Children.Add(txt);

                panItem.Content = grid;

                panorama.Items.Add(panItem);

                slideIndex++;
            }

            StartShow();
        }

        void SetSelectedBorder(int slideIndex)
        {
            if (oldBorder != null)
            {
                oldBorder.BorderThickness = new Thickness(0);
            }

            Border border = imageList.Items[slideIndex - 1] as Border;
            border.BorderThickness = new Thickness(3);

            oldBorder = border;

            imageList.ScrollIntoView(border);
        }

        void SetSlide(int number)
        {
            string url = string.Format("{0}/setSlide/{1}", this._baseUrl, number);
            CallUrl(url, null);
        }

        void StartShow()
        {
            string url = string.Format("{0}/startShow", this._baseUrl);
            CallUrl(url, null);
        }

        void CallUrl(string url, DownloadStringCompletedEventHandler handler)
        {
            WebClient wc = new WebClient();
            wc.Headers[HttpRequestHeader.IfModifiedSince] = DateTime.UtcNow.ToString();

            if (handler != null)
            {
                wc.DownloadStringCompleted += handler;
            }

            wc.DownloadStringAsync(new Uri(url));
        }

        private void panorama_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
            {
                return;
            }

            SlideItemData tagData = (e.AddedItems[0] as PanoramaItem).Tag as SlideItemData;

            int slideIndex = tagData.SlideIndex;
            SetSlide(slideIndex);
            SetSelectedBorder(slideIndex);
        }

        void img_Tap(object sender, GestureEventArgs e)
        {
            SlideItemData tagData = (sender as Image).Tag as SlideItemData;
            int slideIndex = tagData.SlideIndex;

            SetSlide(slideIndex);
            SetSelectedBorder(slideIndex);
        }
    }
}