﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Documents;
using System.Net;
using System.Net.NetworkInformation;
using System.IO;
using System.Threading;
using Microsoft.Win32;
using System.Windows.Threading;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Net.Sockets;

namespace PPTShow
{
    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        MyHttpServer _listener;
        Thread _httpThread;
        PowerpointController _pptController;

        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = this;

            this.Ready = "Not Loaded";
            this.Port = 5022;


            this.IPList = new ObservableCollection<string>();
            PopulateIPList(this.IPList);

            this._pptController = new PowerpointController();
            SetupHttpServer();

            this.Closed += new EventHandler(MainWindow_Closed);
        }

        void MainWindow_Closed(object sender, EventArgs e)
        {
            ReleaseSocketResource();
        }

        private void ReleaseSocketResource()
        {
            try
            {
                if (_listener != null)
                {
                    _listener.Dispose();
                }
            }
            catch { }
            _listener = null;

            try
            {
                if (_httpThread != null)
                {
                    _httpThread.Abort();
                }
            }
            catch { }
            _httpThread = null;
        }

        private void SetupHttpServer()
        {
            _listener = new MyHttpServer(this._pptController, this.Port);
            _httpThread = new Thread(_listener.listen);
            _httpThread.IsBackground = true;
            _httpThread.Start();
        }

        void btnOpenClicked(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openDlg = new OpenFileDialog();
            if (openDlg.ShowDialog() == true)
            {
                string path = openDlg.FileName;
                ProcessDocument(path);
            }
        }

        private void ProcessDocument(string path)
        {
            if (File.Exists(path) == false)
            {
                return;
            }

            if (_pptController.Load(path) == false)
            {
                return;
            }

            this.FilePath = path;
            this.Ready = "Loaded";
        }

        private void PopulateIPList(ObservableCollection<string> list)
        {
            IPAddress[] addrs = Dns.GetHostEntry(Dns.GetHostName()).AddressList;

            foreach (IPAddress ipAddr in addrs)
            {
                if (ipAddr.AddressFamily == AddressFamily.InterNetwork)
                {
                    list.Add(ipAddr.ToString());
                }
            }
        }

        short _port;
        public short Port
        {
            get { return this._port; }

            set
            {
                if (this._port == value)
                {
                    return;
                }

                this._port = value;
                OnPropertyChanged("Port");
            }
        }

        ObservableCollection<string> _iPList;
        public ObservableCollection<string> IPList
        {
            get { return this._iPList; }

            set
            {
                if (this._iPList == value)
                {
                    return;
                }

                this._iPList = value;
                OnPropertyChanged("IPList");
            }
        }

        string _filePath;
        public string FilePath
        {
            get { return this._filePath; }

            set
            {
                if (this._filePath == value)
                {
                    return;
                }

                this._filePath = value;
                OnPropertyChanged("FilePath");
            }
        }

        string _ready;
        public string Ready
        {
            get { return this._ready; }

            set
            {
                if (this._ready == value)
                {
                    return;
                }

                this._ready = value;
                OnPropertyChanged("Ready");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged == null)
            {
                return;
            }

            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }    
    }
}
