﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Interop.PowerPoint;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Web.Script.Serialization;
using Microsoft.Office.Core;

namespace PPTShow
{
    public class PowerpointController
    {
        Application _app;
        Presentation _current;

        const int _slideImageWideth = 480;
        const int _slideImageHeight = 358;

        string _documentAsText;
        public string DocumentAsText
        {
            get { return _documentAsText; }
        }

        public bool Load(string documentPath)
        {
            string tempPath = string.Empty;

            try
            {
                if (_app == null)
                {
                    _app = new Application();
                }

                _app.Visible = Microsoft.Office.Core.MsoTriState.msoTrue;
                var presentation = _app.Presentations;
                _current = presentation.Open(documentPath);

                tempPath = Path.Combine(Path.GetTempPath(), "ShowPresenter");
                tempPath = Path.Combine(tempPath, Guid.NewGuid().ToString());
                Directory.CreateDirectory(tempPath);

                if (Directory.Exists(tempPath) == true)
                {
                    _current.Export(tempPath, "JPG");
                    PPTDocument doc = ReadAll(tempPath);

                    JavaScriptSerializer json = new JavaScriptSerializer();
                    json.MaxJsonLength = Int32.MaxValue;
                    _documentAsText = json.Serialize(doc);
                }

                return true;
            }
            catch
            {
            }
            finally
            {
                try
                {
                    if (Directory.Exists(tempPath) == true)
                    {
                        Directory.Delete(tempPath, true);
                    }
                }
                catch { }
            }

            return false;
        }

        private static int CompareOnlyNumbers(string x, string y)
        {
            string fileNameX = System.IO.Path.GetFileNameWithoutExtension(x);
            string fileNameY = System.IO.Path.GetFileNameWithoutExtension(y);

            int xn = EraseChars(fileNameX);
            int yn = EraseChars(fileNameY);

            return xn.CompareTo(yn);
        }

        private static int EraseChars(string x)
        {
            StringBuilder sb = new StringBuilder();
            foreach (char ch in x)
            {
                if (Char.IsNumber(ch) == true)
                {
                    sb.Append(ch);
                }
            }

            return Int32.Parse(sb.ToString());
        }

        PPTDocument ReadAll(string tempPath)
        {
            int slideNumber = 0;

            List<string> files = new List<string>();
            files.AddRange(Directory.EnumerateFiles(tempPath));
            files.Sort(CompareOnlyNumbers);

            PPTDocument document = new PPTDocument();

            foreach (Slide slide in _current.Slides)
            {
                string note = string.Empty;

                SlideRange nodePath = slide.NotesPage;

                foreach (Microsoft.Office.Interop.PowerPoint.Shape shape in nodePath.Shapes)
                {
                    PpPlaceholderType type = PpPlaceholderType.ppPlaceholderObject;

                    try
                    {
                        type = shape.PlaceholderFormat.Type;
                    }
                    catch { }

                    if (type == PpPlaceholderType.ppPlaceholderBody)
                    {
                        if (shape.HasTextFrame == Microsoft.Office.Core.MsoTriState.msoTrue)
                        {
                            if (shape.TextFrame.HasText == MsoTriState.msoTrue)
                            {
                                note += shape.TextFrame.TextRange.Text;
                            }
                        }
                    }
                }

                PPTPage page = new PPTPage();
                page.Note = note;
                page.ImageAsText = ConvertToImage(files[slideNumber], _slideImageWideth, _slideImageHeight);

                slideNumber++;

                document.List.Add(page);
            }

            document.Width = _slideImageWideth;
            document.Height = _slideImageHeight;
            return document;
        }

        private string ConvertToImage(string path, int width, int height)
        {
            try
            {
                using (Image img = Image.FromFile(path))
                {
                    using (Image thumbnail = img.GetThumbnailImage(width, height, delegate { return false; }, IntPtr.Zero))
                    {
                        MemoryStream ms = new MemoryStream();
                        thumbnail.Save(ms, ImageFormat.Jpeg);
                        ms.Position = 0;

                        return Convert.ToBase64String(ms.GetBuffer(), 0, (int)ms.Length);
                    }
                }
            }
            catch
            {
            }

            return string.Empty;
        }

        public void SetCurrentSlide(int slideNumber)
        {
            try
            {
                _current.SlideShowWindow.View.GotoSlide(slideNumber, MsoTriState.msoTrue);
            }
            catch { }
        }

        public void StartShow(int startSlideNumber)
        {
            object inSlideShow = null;

            try
            {
                inSlideShow = _current.SlideShowWindow;
            }
            catch
            {
            }

            if (inSlideShow == null)
            {
                _current.SlideShowSettings.Run();
                int start = Math.Min(_current.Slides.Count, startSlideNumber);
                _current.SlideShowWindow.View.GotoSlide(start, Microsoft.Office.Core.MsoTriState.msoTrue);
            }
        }
    }
}
